(function () {
    'use strict';

    angular
        .module('ngApp')
        .controller('AparatCtrl', AparatCtrl);

    AparatCtrl.$inject = ['$scope', '$http', '$ionicModal', '$ionicScrollDelegate', '$sce'];

    function AparatCtrl($scope, $http, $ionicModal, $ionicScrollDelegate, $sce) {
        $scope.aparat = [];
        $scope.noMoreItemsAvailable = true;
        $scope.storage = '';

        // Wait until the sl var is init
        $scope.$watch('sl', function () {
            $scope.showAparat(false);
        });

        $scope.showAparat = function (refresh) {
            if (isOnline()) {
                if (!refresh) {
                    $scope.loading = true;
                }
                $http({
                        method: 'GET',
                        noLoading: true,
                        url: url + '/api/v1/widget/route/aparat/getVideos/' + $scope.sl + '?length=10'
                    })
                    .success(function (data) {
                        $scope.$broadcast('scroll.refreshComplete');
                        $scope.storage = 'خطا در اتصال به سرور';
                        switch ($scope.type) {
                            case 'username':
                                $scope.aparat = data.videobyuser || [];
                                break;
                            case 'usercat':
                                $scope.aparat = data.videobyprofilecat || [];
                                break;
                            case 'videos':
                                $scope.aparat = data.videos || [];
                                break;
                        }
                        if ($scope.aparat) {
                            $scope.storage = '';
                            if ($scope.aparat.length == 10) {
                                $scope.noMoreItemsAvailable = false;
                            }
                        }
                        $scope.loading = false;
                    })
                    .error(function () {
                        $scope.$broadcast('scroll.refreshComplete');
                        $scope.storage = 'خطا در اتصال به سرور';
                        $scope.loading = false;
                    });
            } else {
                $scope.loading = false;
                window.plugins.toast.showWithOptions({
                    message: "اتصال اینترنت خود را بررسی کنید",
                    duration: "short",
                    position: "bottom",
                    addPixelsY: -50
                });
            }
        };


        $ionicModal.fromTemplateUrl('aparat-modal.html', {
            scope: $scope,
            animation: 'slide-in-up',
            hardwareBackButtonClose: true
        }).then(function (modal) {
            $scope.modal = modal;
        });

        $scope.openModal = function (index) {
            $scope.caption = $scope.aparat[index].title;
            $scope.description = $scope.aparat[index].description || '';
            $scope.videoId = $scope.aparat[index].uid;
            $scope.videosrc = $sce.trustAsResourceUrl($scope.aparat[index].frame);
            $scope.modal.show();
            $ionicScrollDelegate.$getByHandle('modal').scrollTop();
        };

        $scope.closeModal = function () {
            $scope.modal.hide();
            $scope.caption = $scope.videosrc = '';
        };

        $scope.share = function () {
            var message = $scope.caption;
            var link = 'http://www.aparat.com/v/' + $scope.videoId;

            window.plugins.socialsharing.share(message, null, null, link);
        }

        function strip(html) {
            var tmp = document.createElement("DIV");
            tmp.innerHTML = html;
            return tmp.textContent || tmp.innerText || "";
        }

        $scope.doRefresh = function () {
            $scope.showAparat(true);
        };

        $scope.loadMore = function () {
            if (!$scope.noMoreItemsAvailable) {
                $scope.noMoreItemsAvailable = true;
                var start = $scope.aparat.length;
                if (isOnline()) {
                    $scope.infiniteScrollLoading = true;
                    $http({
                            method: 'GET',
                            noLoading: true,
                            url: url + '/api/v1/widget/route/aparat/getVideos/' + $scope.sl + '/' + start + '?length=10'
                        })
                        .success(function (data) {
                            switch ($scope.type) {
                                case 'username':
                                    data = data.videobyuser || [];
                                    break;
                                case 'usercat':
                                    data = data.videobyprofilecat || [];
                                    break;
                                case 'videos':
                                    data = data.videos || [];
                                    break;
                            }
                            if (data.length == 0 || data.found == 0) {
                                $scope.noMoreItemsAvailable = true;
                            } else {
                                for (var i = 0; i < data.length; i++) {
                                    $scope.aparat.push(data[i]);
                                }
                            }
                            if (data.length < 10) {
                                $scope.noMoreItemsAvailable = true;
                            } else {
                                $scope.noMoreItemsAvailable = false;
                            }
                            $scope.infiniteScrollLoading = false;
                            $scope.$broadcast('scroll.infiniteScrollComplete');
                        })
                        .error(function () {
                            console.error('ERR', err.status);
                            $scope.infiniteScrollLoading = false;
                            $scope.$broadcast('scroll.infiniteScrollComplete');
                        });
                }

            }
        };

        // Cleanup the modal when we're done with it!
        $scope.$on('$destroy', function () {
            $scope.modal.remove();
        });
    }
})();