/**
 * @file Signup widgets Controller
 * @author Hussein Arian <hossein@puzzley.ir>
 * @version 1.7.0
 */
var $signupVersion = 1.7;

(function () {
    'use strict';

    angular.module('ngApp').requires.push('ngFileUpload', 'ionic-datepicker');

    angular
        .module('ngApp')
        .controller('profileCtrl', profileController)
        .controller('signupCtrl', SignupController)
        .controller('LoginCtrl', LoginController)
        .controller('EditCtrl', editProfileController)
        .controller('ticketsCtrl', ticketsContoroller)
        .controller('ticketCtrl', ticketContoroller)
        .factory('Auth', AuthenticationService)
        .factory('UserService', UserService)
        .filter('join', joinFilter)
        .filter('fixConditionalValue', fixConditionalValueFilter);

    /**
     * Profile page Controller
     * show user data
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    profileController.$inject = ['$scope', 'Auth'];
    function profileController($scope, Auth) {
        $scope.version = $signupVersion;

        $scope.logout = function() {
            Auth.Logout();
        }

        $scope.change_password = function(isValid) {
            if (isValid) {
                Auth.ChangePassword($scope.oldPassword, $scope.newPassword, function(response) {
                    if(response.success) {
                        showSuccess('رمزعبور با موفقیت عوض شد', 2400);
                        setTimeout(function () {
                            window.location.hash = homeHash;
                        }, 2500);
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning('فرم را کامل و معتبر وارد کنید');
            }
        }
    }

    /**
     * Edit Profile page Controller
     * show input of user data for update
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    editProfileController.$inject = ['$scope', 'Auth', 'ionicDatePicker', 'ionicTimePicker'];
    function editProfileController($scope, Auth, ionicDatePicker, ionicTimePicker) {

        $scope.edit = angular.copy(Auth.User);
        $scope.editimagesets = $scope.edit.profile_pic != '' ? true : false;
        $scope.model.profile_pic = $scope.edit.profile_pic;

        $scope.submitform = function (isValid) {
            if (isValid) {
                var form_data = $('#editForm').serializeObject();
                var password = form_data.password;
                form_data.profile_pic = $scope.model.profile_pic;
                Auth.Update(form_data, function (response) {
                    if(response.success) {
                        showSuccess('ویرایش با موفقیت انجام شد', 1000);
                        setTimeout(function () {
                            var isUserActive = Boolean(response.payload.userIsActive);
                            if (!isUserActive) {
                                Auth.Logout();
                                if(response.payload.verification == 'by_email') {
                                    $ionicPopup.alert({
                                        title: 'تایید ایمیل',
                                        template: 'ایمیل تایید حساب کاربری برای شما ارسال شد، برای تایید حساب کاربری خود به ایمیل خود مراجعه کنید',
                                        okText: 'تایید',
                                    }).then(function() {
                                        window.location.hash = signupHash;
                                    })
                                } else if(response.payload.verification == 'by_cellphone') {
                                    Auth.username = response.payload.user.username;
                                    Auth.password = password;
                                    window.location.hash = '#/nav/widget/signup/confirmCellphone/'+ signup_sl + '/' + response.payload.user.user_id;
                                }
                            }
                        }, 1100);
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning('فرم را کامل و معتبر وارد کنید');
            }
        }

        var datevalue = false, timevalue = false;
        $scope.openDatePicker = function(model, time) {
            ionicDatePicker.openDatePicker({
                callback: function (unix, dateString) {
                    datevalue = unix;
                    $scope['edit']['extra_data'][model]['value'] = dateString;
                    if(time) {
                        ionicTimePicker.openTimePicker({
                            callback: function (val) {
                            timevalue = val;
                            var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                            $scope['edit']['extra_data'][model]['value'] = dateString + ' - ' + selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                            },
                            inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
                        });
                    }
                },
                inputDate: datevalue != false ? new Date(datevalue) : new Date(),
            });
        };

        $scope.openTimePicker = function(model){
            ionicTimePicker.openTimePicker({
                callback: function (val) {
                    timevalue = val;
                    var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                    $scope['edit']['extra_data'][model]['value'] = selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                },
                inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
            });
        };

        $scope.showChangePassword = true;
    }

    /**
     * signup form Controller
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    SignupController.$inject = ['$scope', '$timeout', 'Auth', 'UserService', '$ionicPopup', '$interval', 'ionicDatePicker'];
    function SignupController($scope, $timeout, Auth, UserService, $ionicPopup, $interval, ionicDatePicker) {
        $scope.imagesets = false;
        $scope.model = {};
        $scope.confirmMetode = '';
        $scope.time = 90;

        $scope.submit = function (isValid) {
            if (isValid) {
                var form_data = $('#signupForm').serializeObject();
                form_data.profile_pic = $scope.model.profile_pic;

                UserService.Register(form_data)
                    .success(function(result) {
                        if (result.success) {
                            showSuccess('ثبت نام شما با موفقیت انجام شد', 1000);
                            setTimeout(function () {
                                if (result.payload.isactive) {
                                    Auth.Login(result.payload.username, $scope.model.password, function(response) {
                                        if(response.success)
                                            window.location.hash = homeHash;
                                    });
                                } else {
                                    if(result.payload.verification == 'by_email') {
                                        $ionicPopup.alert({
                                            title: 'تایید ایمیل',
                                            template: 'ایمیل تایید حساب کاربری برای شما ارسال شد، برای تایید حساب کاربری خود به ایمیل خود مراجعه کنید',
                                            okText: 'تایید',
                                        }).then(function() {
                                            window.location.hash = signupHash;
                                        })
                                    } else if(result.payload.verification == 'by_cellphone') {
                                        Auth.username = result.payload.username;
                                        Auth.password = $scope.model.password;
                                        window.location.hash = '#/nav/widget/signup/confirmCellphone/'+ signup_sl + '/' + result.payload.user_id;
                                    }
                                }
                            }, 1100);
                        } else {
                            showWarning(result.error);
                        }
                    })
                    .error(function (result, statusCode) {
                        if (statusCode == -1) {
                            showWarning('عدم برقراری ارتباط لطفا مجددا تلاش کنید');
                        } else {
                            checkYourInternet();
                        }
                    });
            } else {
                showWarning('فرم را کامل و معتبر وارد کنید');
            }
        };

        $scope.validateFile = function(files) {
            if (files != null) {
                var file = files[0];
                if (window.FileReader != null && typeof file != "undefined" && typeof file.type != "undefined") {
                    if (file.size < 5 * 1024 * 1024) {
                        $timeout(function() {
                            var fileReader = new FileReader();
                            fileReader.readAsDataURL(file); // convert the image to data url.
                            fileReader.onload = function(e) {
                                $timeout(function() {
                                    if (file.type.indexOf('image') > -1) {
                                        $scope.imagesets = true;
                                        $scope.model.profile_pic = e.target.result;
                                    } else {
                                        showWarning("نوع فایل مجاز نیست");
                                    }
                                });
                            }
                        });
                    } else {
                        showWarning("حداکثر حجم مجاز 5m می باشد.");
                    }
                } else {
                    $scope.imagesets = false;
                    showWarning("فایل معتبر نمی باشد");
                }
            }
        };

        $scope.submitConfirm = function (isValid) {
            if (isValid && $scope.code != '') {
                Auth.Confirm($scope.code, $scope.userId, function (response) {
                    if (response.success) {
                        showSuccess('ثبت اطلاعات و فعالسازی با موفقیت انجام شد.',2000);
                        $timeout(function () {
                            Auth.Login(Auth.username, Auth.password, function (response) {
                                if (response.success) {
                                    window.location.hash = homeHash;
                                } else {
                                    showWarning('خطا در ورود به سیستم');
                                    window.location.hash = signupHash;
                                }
                            });
                        }, 2200);
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning('فرم را کامل و معتبر وارد کنید');
            }
        }

        $scope.reSendCode = function() {
            if($scope.time <= 5) {
                Auth.ReSendConfirm($scope.userId, function(response) {
                    if (response.success) {
                        showSuccess('کد مجددا ارسال شد');
                        $scope.startTimer();
                    } else {
                        showWarning(response.message);
                    }
                })
            }
        }

        $scope.startTimer = function() {
            $scope.time = 90;
            $interval(function() {
                if($scope.time > 0) {
                    $scope.time--;
                } else {
                    $interval.cancel(stop);
                }
            }, 1000);
        }

        var datevalue = false, timevalue = false;
        $scope.openDatePicker = function(model, time) {
            ionicDatePicker.openDatePicker({
                callback: function (unix, dateString) {
                    datevalue = unix;
                    $scope['model'][model] = dateString;
                    if(time) {
                        ionicTimePicker.openTimePicker({
                            callback: function (val) {
                            timevalue = val;
                            var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                            $scope['model'][model] = dateString + ' - ' + selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                            },
                            inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
                        });
                    }
                },
                inputDate: datevalue != false ? new Date(datevalue) : new Date(),
            });
        };

        $scope.openTimePicker = function(model){
            ionicTimePicker.openTimePicker({
                callback: function (val) {
                    timevalue = val;
                    var selectedTime = typeof (val) === 'undefined' ? new Date() : new Date(val * 1000);
                    $scope['model'][model] = selectedTime.getUTCHours() + ':' + selectedTime.getUTCMinutes();
                },
                inputTime: timevalue != false ? timevalue : (new Date()).getHours() * 60 * 60,
            });
        };
    }

    /**
     * Login form Controller
     * @type {Controller}
     * @param {Auth} Authentication Service
     */
    LoginController.$inject = ['$scope', 'Auth', '$ionicPopup', '$ionicScrollDelegate', '$timeout'];
    function LoginController($scope, Auth, $ionicPopup, $ionicScrollDelegate, $timeout) {
        $timeout(function () {
            $ionicScrollDelegate.resize();
        }, 100);
        $scope.submit = function(isValid) {
            if(isValid) {
                Auth.Login($scope.user.username ,$scope.user.password, function (response) {
                    if(response.success){
                        window.location.hash = homeHash;
                    } else {
                        showWarning(response.message);
                    }
                });
            } else {
                showWarning('فرم را کامل و معتبر وارد کنید');
            }
        };

        $scope.forget_password = function(isValid) {
            if(isValid) {
                Auth.ForgetPassword($scope.user_identity , function (response) {
                    if(response.success){
                        $ionicPopup.alert({
                            title: '',
                            template: response.message,
                            buttons: [{
                                text: 'تایید',
                                type: 'button-default'
                            }]
                        });
                        window.location.hash = signupHash;
                    }
                });
            } else {
                showWarning('فرم را کامل و معتبر وارد کنید');
            }
        }
    }

    /**
      * Tickets Controller
      * ticket list view controller
      */
    ticketsContoroller.$inject = ['$scope', '$http'];
    function ticketsContoroller($scope, $http) {
		var vm = this;
		vm.loadMore = loadMore;
		vm.refresh = refresh;
		vm.noMoreItemsAvailable = true;
		vm.infiniteScrollLoading = false;
		vm.message = '';
		var length = 20;

		$scope.$watch('sl', function () {
			getTickets();
		});
		
		function getTickets() {
			$http.post(url + '/api/v1/widget/post/signup/getTickets', {
				sl: $scope.sl,
				length: length
			})
				.success(function (result) {
					if (result.success) {
                        vm.tickets = result.payload.tickets;
                        vm.message = result.message;
						if (result.payload.tickets.length == length) {
							vm.noMoreItemsAvailable = false;
						}
					}
					$scope.$broadcast('scroll.refreshComplete');
				})
				.error(function () {
					checkYourInternet();
					$scope.$broadcast('scroll.refreshComplete');
				});
		}

		function refresh() {
			getTickets();
		}
		
		function loadMore() {
			if (!vm.noMoreItemsAvailable) {
				vm.noMoreItemsAvailable = true;
				var start = vm.tickets.length;
				if (isOnline()) {
					vm.infiniteScrollLoading = true;
					var data = {
						sl: $scope.sl,
						start: start,
						length: length
					};
					$http({
							method: 'POST',
							noLoading: true,
							url: url + '/api/v1/widget/post/signup/getTickets',
							data: data
						})
						.success(function (result) {
							if (result.success) {
								for (var i = 0; i < result.payload.tickets.length; i++) {
									vm.tickets.push(result.payload.tickets[i]);
								}
								vm.noMoreItemsAvailable = result.payload.tickets.length < length;
							}
							vm.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						})
						.error(function () {
							console.error('ERR', err.status);
							vm.infiniteScrollLoading = false;
							$scope.$broadcast('scroll.infiniteScrollComplete');
						});
				}
			} else {
				return;
			}
		}
    }
    

    /**
     * Ticket Controller
     * single ticket view Controller
     */
    ticketContoroller.$inject = ['$scope', '$rootScope', '$http'];
    function ticketContoroller($scope, $rootScope, $http) {
		var vm = this;
		vm.ticket = {};
		vm.sendTicket = sendTicket;
		vm.usertickets = [];

        // on init
        $scope.$watch('ticket_id', function () {
            if ($scope.ticket_id) {
                $rootScope['ticket' + $scope.ticket_id] = 'visited';
                $rootScope.user.unread_tickets_count -= $scope.unread_ticket_count;
                $rootScope.user.unread_tickets_count = Math.max($rootScope.user.unread_tickets_count == undefined ? 0 : $rootScope.user.unread_tickets_count, 0);
            }
		});

		function sendTicket(valid) {
		    if (valid) {
		        var fd = new FormData();
		        for (var key in vm.ticket) {
		            fd.append(key, vm.ticket[key]);
		        }
		        fd.append('sl', $scope.sl);

		        $http.post(url + '/api/v1/widget/post/signup/postTicket', fd, {
		                withCreadential: true,
		                headers: {
		                    'Content-Type': undefined,
		                },
		                transformRequest: angular.identity
		            })
		            .success(function (result) {
		                if (result.success) {
		                    vm.ticket = {
		                        title: '',
		                        content: '',
		                        attach: ''
                            };
                            vm.usertickets.push(result.payload.ticket);
                            if (typeof result.payload.sl != "undefined")
                                $scope.sl = result.payload.sl;
                            showToast('پیام شما با موفقیت ثبت شد');
                            $scope.fm.$setPristine();
                            $scope.fm.$setUntouched();
		                } else {
		                    showToast(result.message);
		                }
		            })
		            .error(function () {
		                showToast('خطا در ارسال، اتصال اینترنت خود را بررسی کنید');
		            });
		    } else {
		        showToast('فرم را کامل و معتبر وارد کنید');
		        return;
		    }
		}
	}
    
    /**
     * service for make http call for login and Logut
     * @type {factory}
     * @param {UserService} UserService : keeps user data
     * @return {object} AuthenticationService
     */
    AuthenticationService.$inject = ['$rootScope', '$http', '$timeout', 'localStorage', 'UserService'];
    function AuthenticationService($rootScope, $http, $timeout, localStorage, UserService) {
        var Auth = {};
        Auth.username = '';
        Auth.password = '';
        Auth.User = {isLogin : false, accessible_pages : default_access_page};
        Auth.Login = Login;
        Auth.Check = Check;
        Auth.Logout = Logout;
        Auth.CanAccess = CanAccess;
        Auth.SetCredentials = SetCredentials;
        Auth.Update = Update;
        Auth.Confirm = Confirm;
        Auth.ReSendConfirm = ReSendConfirm;
        Auth.ForgetPassword = ForgetPassword;
        Auth.ChangePassword = ChangePassword;
        Auth.init = init;

        (function(){
            Auth.init();
        })();

        return Auth;

        function init() {
            Auth.User = localStorage.all('user');
            $rootScope.user = Auth.User;
            if(Auth.User.isLogin){
                $http.defaults.headers.common['X-auth-token'] = Auth.User.token;
                $timeout(function () {
                    UserService.UpdateUserInfo().success(function (result) {
                        if (result.success) {
                            Auth.SetCredentials(result.payload.user, result.payload.token);
                        }
                    });
                }, 4000);
            } else {
                Auth.User.isLogin = false;
                Auth.User.accessible_pages = default_access_page;
                $timeout(function () {
                    UserService.UpdateDefaultAccessPage().success(function (result) {
                        if (result.success) {
                            Auth.User.accessible_pages = result.payload.accessible_pages;
                        }
                    });
                }, 2000);
            }
        }

        function Login(username, password, callback) {
            var response;
            UserService.LoginWithUsername(username, password)
                .success(function (result) {
                    if (result.success) {
                        Auth.SetCredentials(result.payload.user, result.payload.token);
                        response = { success: true };
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function (result){
                    callback({ success: false, message: 'خطا در برقراری ارتباط با سرور' });
                    console.error(result);
                });
        }

        function Check() {
            return Auth.User.isLogin;
        }
        function CanAccess(pageid) {
            return (
                typeof pageid !== 'undefined') &&
                (Auth.User.accessible_pages.includes(Number(pageid)) || (Auth.User.accessible_pages.includes(String(pageid)))
            );
        }

        function Logout() {
            UserService.Logout();
            Auth.User.isLogin = false;
            Auth.User.token = '';
            Auth.User.accessible_pages = default_access_page;
            localStorage.save(Auth.User,'user');
        }

        function SetCredentials(user, token) {
            Auth.User = user;
            Auth.User.isLogin = true;
            Auth.User.token = token;
            $rootScope.user = Auth.User;
            $http.defaults.headers.common['X-auth-token'] = token;
            localStorage.save(Auth.User,'user');
        }

        function Update(user, callback) {
            var response;
            UserService.Update(user)
                .success(function (result) {
                    result.payload = typeof result.payload === 'undefined' ? [] : result.payload;
                    if (result.success) {
                        Auth.SetCredentials(result.payload.user, result.payload.token);
                        response = { success: true, payload: result.payload };
                    } else {
                        response = { success: false, message: result.error, payload: result.payload };
                    }
                    callback(response);
                })
                .error(function (result, statusCode) {
                    result.payload = typeof result.payload === 'undefined' ? [] : result.payload;
                    var message = statusCode == -1 ? 'خطا در ارسال اطلاعات' : 'خطا در بروزرسانی';
                    callback({ success: false, message: message, payload: result.payload });
                });
        }

        function Confirm(code, user_id, callback) {
            var response;
            UserService.ConfirmCellphone(code, user_id)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true };
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function (){
                    callback({ success: false, message: 'خطا در ارسال اطلاعات' });
                });
        }

        function ReSendConfirm(user_id, callback) {
            var response;
            UserService.resendconfirmcode(user_id)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true };
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function () {
                    callback({ success: false, message: 'خطا در ارسال اطلاعات' });
                });
        }

        function ForgetPassword(user_id, callback) {
            var response;
            UserService.ForgetPassword(user_id)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true, message: result.payload.message };
                    }
                    callback(response);
                })
                .error(function (){
                    checkYourInternet();
                    callback({ success: false, message: 'خطا در ارسال اطلاعات' });
                });
        }

        function ChangePassword(oldPass, newPass, callback) {
            var response;
            UserService.ChangePassword(oldPass, newPass)
                .success(function (result) {
                    if (result.success) {
                        response = { success: true};
                    } else {
                        response = { success: false, message: result.error };
                    }
                    callback(response);
                })
                .error(function (result){
                    callback({ success: false, message: 'خطا در ارسال اطلاعات' });
                });
        }
    }

    /**
     * service for make API call to users api
     * @type {factory}
     * @param {$http}
     * @return {object} UserService
     */
    UserService.$inject = ['$http'];
    function UserService($http) {
        var service = {};

        service.LoginWithUsername = LoginWithUsername;
        service.Register = Register;
        service.Update = Update;
        service.ConfirmCellphone = ConfirmCellphone;
        service.resendconfirmcode = resendconfirmcode;
        service.UpdateUserInfo = UpdateUserInfo;
        service.Logout = Logout;
        service.ForgetPassword = ForgetPassword;
        service.ChangePassword = ChangePassword;
        service.UpdateDefaultAccessPage = UpdateDefaultAccessPage;

        return service;

        function LoginWithUsername(username, password) {
            return $http.post(url + '/api/v1/widget/post/signup/Login', {
                        username : username,
                        password : password,
                        phone_id : phone_id,
                        sl : signup_sl
                    });
        }

        function Register(user) {
            return $http.post(url + '/api/v1/widget/post/signup/Register', user);
        }

        function Update(user) {
            return $http.post(url + '/api/v1/widget/post/signup/editUser', user);
        }

        function ConfirmCellphone(code, user_id) {
            return $http.post(url + '/api/v1/widget/post/signup/confirmByCellphone', {
                        code : code,
                        user_id : user_id,
                        sl : signup_sl
                    });
        }

        function resendconfirmcode(user_id) {
            return $http.post(url + '/api/v1/widget/post/signup/resendConfirmCodeToCellphone', {
                        user_id : user_id,
                        sl : signup_sl
                    });
        }

        function UpdateUserInfo() {
            return $http.post(url + '/api/v1/widget/post/signup/updateUserInfo', {
                        sl : signup_sl
                    },{
                        noLoading : true
                    });
        }

        function Logout() {
            return $http.post(url + '/api/v1/widget/post/signup/logout', {
                        sl : signup_sl
                    },{
                        noLoading : true
                    });
        }

        function ForgetPassword(user_id) {
            return $http.post(url + '/api/v1/widget/post/signup/resetPassword', {
                        sl : signup_sl,
                        user_identity : user_id
                    });
        }

        function ChangePassword(oldPass, newPass) {
            return $http.post(url + '/api/v1/widget/post/signup/changePassword', {
                        sl : signup_sl,
                        old_password : oldPass,
                        new_password : newPass
                    });
        }

        function UpdateDefaultAccessPage() {
            return $http.post(url + '/api/v1/widget/post/signup/updateDefaultAccessPage', {
                sl: signup_sl
            }, {
                noLoading: true
            });
        }

    }
    /**
     * filter to convert conditional values to miningfull string
     * @type {filter}
     * @param {string} string string to convert
     * @return {function} filter
     */
    function fixConditionalValueFilter() {
        return function fixConditionalValue(string) {
            if (Array.isArray(string) || typeof string == 'undefined') {
                return string;
            }
            var originalString = string.toString();
            try {
                string = string.toLowerCase();
            } catch (e) {
                string = '';
            }
            return (
                    string == 'on'    || 
                    string == 'true'  || 
                    string == '1'
                   )    ? 'بله' :
                   (
                    string == 'off'   || 
                    string == 'false' || 
                    string == '0'
                   )   ? 'خیر' : originalString;
        };
    }
    /**
     * filter to join array data as string with giving prop
     * @type {filter}
     * @param {array} array string or array to convert
     * @param {sprator} string sprator sign
     * @param {prop} string prop of array to show
     * @return {function} filter
     */
    function joinFilter() {
        return function join(array, separator, prop) {
            if (!Array.isArray(array)) {
                return array; // if not array return original
            }

            return (!angular.isUndefined(prop) ? array.map(function (item) {
                return item[prop];
            }) : array).join(separator);
        };
    }
})();

document.addEventListener("deviceready", function(){
	phone_id = device.uuid;
});

$.fn.serializeObject = function () {
    var o = {};
    var a = this.serializeArray();
    // console.log(a);
    $.each(a, function () {
        if (o[this.name] !== undefined) {
            if (!o[this.name].push) {
                o[this.name] = [o[this.name]];
            }
            o[this.name].push(this.value || '');
        } else {
            o[this.name] = this.value || '';
        }
    });
    return o;
};